import { TimeInSeconds } from "../midi/types";
export interface IAndySoundNode {
    readonly node?: AudioNode;
    readonly nodeInput?: AudioNode;
}
export declare class Sound implements IAndySoundNode {
    buffer: AudioBuffer;
    loop: boolean;
    loopStart: number;
    loopEnd: number;
    isInContext: boolean;
    isPlaying: boolean;
    constructor(buffer: AudioBuffer);
    private source;
    volume: GainNode | undefined;
    panner: PannerNode | undefined;
    setContext(context: AudioContext): void;
    get nodeInput(): AudioNode | undefined;
    get node(): AudioNode | undefined;
    play(time?: number): void;
    stop(): void;
    destroy(): void;
}
export declare class SoundSystem implements IAndySoundNode {
    static addSoundToContext(context: AudioContext, sound: Sound): void;
    static soundContext: AudioContext;
    static masterGainNode: GainNode;
    static analyser: AnalyserNode;
    constructor();
    context(): AudioContext;
    masterGainNode(): GainNode;
    get nodeInput(): AudioNode | undefined;
    reboot(): void;
    getAnalyser(): AnalyserNode;
    volume(): number;
    volumeInDB(): number;
    setVolumeInDB(decibel: number): void;
    adjustVolume(by: number): void;
    play(sound: Sound, multiple?: boolean, time?: TimeInSeconds): void;
    stop(sound: Sound): void;
    currentTime(): TimeInSeconds;
    decodeSound(data: ArrayBuffer): Promise<AudioBuffer>;
    resumePlay(): Promise<void>;
    private playSound;
    private handleStateChange;
}
export declare const loadSound: (url: string, soundSystem: SoundSystem) => Promise<Sound>;
//# sourceMappingURL=audio.d.ts.map