export class BinaryFile {
    constructor(data) {
        this.buffer = data;
        this.dataView = new DataView(this.buffer, 0);
        this.offset = 0;
    }
    tell() {
        return this.offset;
    }
    seek(offset) {
        this.offset = offset;
    }
    skip(bytes) {
        this.offset += bytes;
    }
    eof() {
        return this.offset >= this.buffer.byteLength;
    }
    slice(offset, length) {
        return new BinaryFile(this.buffer.slice(offset, offset + length));
    }
    read(length) {
        const result = new BinaryFile(this.buffer.slice(this.offset, this.offset + length));
        this.offset += length;
        return result;
    }
    readString(length) {
        let result = '';
        let terminated = false;
        for (let i = 0; i < length; i++) {
            const byte = this.readUInt8();
            if (byte === 0x0) {
                terminated = true;
            }
            if (!terminated) {
                result += String.fromCharCode(byte);
            }
        }
        return result;
    }
    readCString() {
        let result = '';
        for (let i = 0; i < 128; i++) {
            const byte = this.readUInt8();
            if (byte === 0x0) {
                return result;
            }
            result += String.fromCharCode(byte);
        }
        return result;
    }
    readChars(count) {
        return this.readBytes(count);
    }
    readBytes(count) {
        let result = new Uint8Array(count);
        for (let i = 0; i < count; i++) {
            const byte = this.readUInt8();
            result[i] = byte;
        }
        return result;
    }
    readSBytes(count) {
        let result = new Int8Array(count);
        for (let i = 0; i < count; i++) {
            const byte = this.readInt8();
            result[i] = byte;
        }
        return result;
    }
    readUInt8() {
        return this.dataView.getUint8(this.offset++);
    }
    readInt8() {
        return this.dataView.getInt8(this.offset++);
    }
    readUInt16(little = true) {
        const result = this.dataView.getUint16(this.offset, little);
        this.offset += 2;
        return result;
    }
    readInt16(little = true) {
        const result = this.dataView.getInt16(this.offset, little);
        this.offset += 2;
        return result;
    }
    readUInt32(little = true) {
        const result = this.dataView.getUint32(this.offset, little);
        this.offset += 4;
        return result;
    }
    readInt32(little = true) {
        const result = this.dataView.getInt32(this.offset, little);
        this.offset += 4;
        return result;
    }
    readFloat(little = true) {
        const result = this.dataView.getFloat32(this.offset, little);
        this.offset += 4;
        return result;
    }
    close() {
        this.offset = -1;
    }
}
