let system;
let isPlaying = false;
let lookahead = 25.0;
let scheduleAheadTime = 0.1;
let nextNoteTime = 0.0;
let timerWorker;
let beatInfo;
let totalTick = 0;
function nextTick(tempo, sig) {
    const secondsPerQuaterNote = (60000 / (tempo.bpm * tempo.ppq));
    const millisecondsPerQuarterNote = secondsPerQuaterNote / 1000;
    nextNoteTime += millisecondsPerQuarterNote;
}
export const toSignature = (sig) => {
    const p = sig.split("/");
    return createSignature(parseInt(p[0]), parseInt(p[1]));
};
export const createSignature = (numerator, denominator, quarter32nd, metroTicks = 24) => {
    return {
        noteBeats: numerator,
        perMeasure: denominator,
        ticksInMetronome: metroTicks,
        toQuarter32nd: quarter32nd !== null && quarter32nd !== void 0 ? quarter32nd : numerator * 2,
    };
};
export const createTempo = (ppq, usPerQuarter) => {
    const tempo = {
        ppq: ppq,
        tempo: usPerQuarter,
        bpm: 60000000 / ((usPerQuarter > 0) ? usPerQuarter : 0),
    };
    return tempo;
};
function scheduler(tempo, sig, perTick) {
    while (nextNoteTime < (system.currentTime() + scheduleAheadTime)) {
        if (totalTick % tempo.ppq === 0) {
            beatInfo.beat++;
            beatInfo.tick = 0;
        }
        else {
            beatInfo.tick += 1;
        }
        if (beatInfo.beat === (sig.noteBeats + 1)) {
            beatInfo.measure++;
            beatInfo.beat = 1;
            beatInfo.tick = 0;
        }
        for (let l = 0; l < perTick.tickListeners.length; l++) {
            [tempo, sig] = perTick.tickListeners[l](totalTick, nextNoteTime, beatInfo);
        }
        nextTick(tempo, sig);
        totalTick++;
    }
}
export function playHeadToggle() {
    isPlaying = !isPlaying;
    if (isPlaying) {
        nextNoteTime = system.currentTime();
        timerWorker.postMessage("start");
    }
    else {
        timerWorker.postMessage("stop");
    }
}
export function playHeadTo(time) {
    totalTick = time;
}
export function playHeadRewind() {
    nextNoteTime = system.currentTime();
    beatInfo = {
        measure: 1,
        beat: 1,
        tick: 0,
    };
    totalTick = 0;
}
export function createPlayHeadClock(sys, tempo, sig, perTick) {
    system = sys;
    timerWorker = new Worker("metronome.worker.js");
    playHeadRewind();
    timerWorker.onmessage = function (e) {
        if (e.data === "tick") {
            scheduler(tempo, sig, perTick);
        }
        else {
            console.log("message: " + e.data);
        }
    };
    timerWorker.postMessage({ "interval": lookahead });
}
