export declare enum Format {
    Single = 0,
    MultiSync = 1,
    MultiAsync = 2
}
declare type MetaSubtype = 'NoteOn' | 'NoteOff' | 'NoteAfterTouch' | 'Controller' | 'ProgramChange' | 'ChannelAfterTouch' | 'PitchBend';
declare type EventSubtype = 'SequenceNumber' | 'Text' | 'CopyrightNotice' | 'TrackName' | 'InstrumentName' | 'Lyrics' | 'Marker' | 'CuePoint' | 'MidiChannelPrefix' | 'EndOfTrack' | 'SetTempo' | 'SMPTEOffset' | 'TimeSignature' | 'KeySignature' | 'SequencerSpecific' | 'Unknown' | MetaSubtype;
declare type MidiEvent = {
    raw: number;
    time: number;
    abstime: number;
    type: 'Meta' | 'SysEx' | 'Channel';
    channel?: number;
    subtype?: EventSubtype;
    data: Array<number | string | Uint8Array>;
};
declare type Track = {
    name: string;
    events: Array<MidiEvent>;
};
export declare type Marker = {
    time: number;
    abstime: number;
};
export declare class MIDI {
    private data;
    ticksPerQuarterNote: number;
    format: Format;
    numberOfTracks: number;
    tracks: Array<Track>;
    markers: Map<string, Marker>;
    constructor(data: ArrayBuffer);
    private readHeader;
    private readTrack;
    private metaEvents;
    private sysexEvents;
    private midiEvents;
    private readVariableLen;
}
export declare const loadMidi: (url: string) => Promise<MIDI>;
export {};
//# sourceMappingURL=song.d.ts.map