export declare type Effect = {
    command: number;
    argX: number;
    argY: number;
};
export declare type MidiInputNote = number;
export declare type TimeInSeconds = number;
export declare function noteByMidi(value: number): string;
export declare const NoteToMidi: Map<string, number>;
export declare const MidiToFreq: Array<number>;
export declare type WaveTypes = "sine" | "square" | "sawtooth" | "triangle" | "custom";
export declare type VCO = {
    type: WaveTypes;
    frequency?: number;
    freqOffset?: number;
    freqMult?: number;
    detune: number;
};
export declare type VCA = {
    gain?: number;
    gainOffset?: number;
};
export declare type ADSR = {
    attack: number;
    decay: number;
    sustain: number;
    release: number;
};
export declare type Filter = {
    type: 'lowpass' | 'highpass' | 'bandpass' | 'lowshelf' | 'highshelf' | 'peaking' | 'notch' | 'allpass';
    frequency: number;
    detune?: number;
    q: number;
    gain: number;
};
export declare type Compressor = {
    threshold: number;
    knee: number;
    ratio: number;
    attack: TimeInSeconds;
    release: TimeInSeconds;
};
export declare type Voice = {
    vco: VCO;
    vca: VCA;
};
export declare type Patch = {
    len: TimeInSeconds;
    vca?: VCA;
    voices: Array<Voice>;
    filter?: Filter;
    compressor?: Compressor;
    adsr?: ADSR;
};
export declare enum GeneralMIDIPercussion {
    HighQ = 27,
    Slap = 28,
    ScratchPush = 29,
    ScratchPull = 30,
    Sticks = 31,
    SquareClick = 32,
    MetronomeClick = 33,
    MetronomeBell = 34,
    BassDrum2 = 35,
    BassDrum1 = 36,
    SideStick = 37,
    SnareDrum1 = 38,
    HandClap = 39,
    SnareDrum2 = 40,
    LowTom2 = 41,
    ClosedHiHat = 42,
    LowTom1 = 43,
    PedalHiHat = 44,
    MidTom2 = 45,
    OpenHiHat = 46,
    MidTom1 = 47,
    HighTom2 = 48,
    CrashCymbal1 = 49,
    HighTom1 = 50,
    RideCymbal1 = 51,
    ChineseCymbal = 52,
    RideBell = 53,
    Tambourine = 54,
    SplashCymbal = 55,
    Cowbell = 56,
    CrashCymbal2 = 57,
    VibraSlap = 58,
    RideCymbal2 = 59,
    HighBongo = 60,
    LowBongo = 61,
    MuteHighConga = 62,
    OpenHighConga = 63,
    LowConga = 64,
    HighTimbale = 65,
    LowTimbale = 66,
    HighAgogo = 67,
    LowAgogo = 68,
    Cabasa = 69,
    Maracas = 70,
    ShortWhistle = 71,
    LongWhistle = 72,
    ShortGuiro = 73,
    LongGuiro = 74,
    Claves = 75,
    HighWoodBlock = 76,
    LowWoodBlock = 77,
    MuteCuica = 78,
    OpenCuica = 79,
    MuteTriangle = 80,
    OpenTriangle = 81,
    Shaker = 82,
    JingleBell = 83,
    Belltree = 84,
    Castanets = 85,
    MuteSurdo = 86,
    OpenSurdo = 87
}
export declare enum GeneralMidiInstruments {
    AcousticGrand = 1,
    BrightAcoustic = 2,
    ElectricGrand = 3,
    HonkyTonk = 4,
    ElectricPiano1 = 5,
    ElectricPiano2 = 6,
    Harpsichord = 7,
    Clav = 8,
    Celesta = 9,
    Glockenspiel = 10,
    MusicBox = 11,
    Vibraphone = 12,
    Marimba = 13,
    Xylophone = 14,
    TubularBells = 15,
    Dulcimer = 16,
    DrawbarOrgan = 17,
    PercussiveOrgan = 18,
    RockOrgan = 19,
    ChurchOrgan = 20,
    ReedOrgan = 21,
    Accoridan = 22,
    Harmonica = 23,
    TangoAccordian = 24,
    AcousticGuitar_nylon = 25,
    AcousticGuitar_steel = 26,
    ElectricGuitar_jazz = 27,
    ElectricGuitar_clean = 28,
    ElectricGuitar_muted = 29,
    OverdrivenGuitar = 30,
    DistortionGuitar = 31,
    GuitarHarmonics = 32,
    AcousticBass = 33,
    ElectricBass_finger = 34,
    ElectricBass_pick = 35,
    FretlessBass = 36,
    SlapBass1 = 37,
    SlapBass2 = 38,
    SynthBass1 = 39,
    SynthBass2 = 40,
    Violin = 41,
    Viola = 42,
    Cello = 43,
    Contrabass = 44,
    TremoloStrings = 45,
    PizzicatoStrings = 46,
    OrchestralStrings = 47,
    Timpani = 48,
    StringEnsemble1 = 49,
    StringEnsemble2 = 50,
    SynthStrings1 = 51,
    SynthStrings2 = 52,
    ChoirAahs = 53,
    VoiceOohs = 54,
    SynthVoice = 55,
    OrchestraHit = 56,
    Trumpet = 57,
    Trombone = 58,
    Tuba = 59,
    MutedTrumpet = 60,
    FrenchHorn = 61,
    BrassSection = 62,
    SynthBrass1 = 63,
    SynthBrass = 64,
    SopranoSax = 65,
    AltoSax = 66,
    TenorSax = 67,
    BaritoneSax = 68,
    Oboe = 69,
    EnglishHorn = 70,
    Bassoon = 71,
    Clarinet = 72,
    Piccolo = 73,
    Flute = 74,
    Recorder = 75,
    PanFlute = 76,
    BlownBottle = 77,
    Shakuhachi = 78,
    Whistle = 79,
    Ocarina = 80,
    Lead1_square = 81,
    Lead2_sawtooth = 82,
    Lead3_calliope = 83,
    Lead4_chiff = 84,
    Lead5_charang = 85,
    Lead6_voice = 86,
    Lead7_fifths = 87,
    Lead8_bass_lead = 88,
    Pad1_new_age = 89,
    Pad2_warm = 90,
    Pad3_polysynth = 91,
    Pad4_choir = 92,
    Pad5_bowed = 93,
    Pad6_metallic = 94,
    Pad7_halo = 95,
    Pad8_sweep = 96,
    FX1_rain = 97,
    FX2_soundtrack = 98,
    FX3_crystal = 99,
    FX4_atmosphere = 100,
    FX5_brightness = 101,
    FX6_goblins = 102,
    FX7_echoes = 103,
    FX8_sci_fi = 104,
    Sitar = 105,
    Banjo = 106,
    Shamisen = 107,
    Koto = 108,
    Kalimba = 109,
    Bagpipe = 110,
    Fiddle = 111,
    Shanai = 112,
    TinkleBell = 113,
    Agogo = 114,
    SteelDrums = 115,
    Woodblock = 116,
    TaikoDrum = 117,
    MelodicTom = 118,
    SynthDrum = 119,
    ReverseCymbal = 120,
    GuitarFretNoise = 121,
    BreathNoise = 122,
    Seashore = 123,
    BirdTweet = 124,
    TelephoneRing = 125,
    Helicopter = 126,
    Applause = 127,
    Gunshot = 128
}
//# sourceMappingURL=types.d.ts.map