import { DURATION_CLICK_PAD, Sound } from "../andy";
import { NoteTable, PAL_CLOCK } from "./types";
import { MOD } from "./song";
export const samplePlayer = (system, mod, sample) => {
    return (note, time, track) => {
        var _a, _b, _c, _d, _e;
        const period = [...NoteTable.entries()]
            .filter(({ 1: v }) => v === note)
            .map(([k]) => k);
        const sampBuffer = mod.sampleBuffers[sample];
        let tsample;
        if (sampBuffer.length) {
            const buff = system.context().createBuffer(1, sampBuffer.length, Math.floor(PAL_CLOCK / period[0]));
            const channel = buff.getChannelData(0);
            for (let i = 0; i < buff.length; i++) {
                channel[i] = sampBuffer[i] / 128;
            }
            const meta = mod.samples[sample];
            tsample = new Sound(buff);
            const samplen = buff.duration;
            tsample.loop = false;
            tsample.loopStart = 0;
            tsample.loopEnd = samplen;
            if (meta.repeatPoint > 0) {
                console.log("loop", meta.repeatLength, meta.repeatPoint);
                tsample.loop = true;
                const repeatStart = (meta.repeatPoint / sampBuffer.length) * samplen;
                const repeatEnd = ((meta.repeatPoint + meta.repeatLength) / sampBuffer.length) * samplen;
                tsample.loopStart = repeatStart;
                tsample.loopEnd = samplen;
            }
            tsample.setContext(system.context());
            (_a = tsample.volume) === null || _a === void 0 ? void 0 : _a.gain.setValueAtTime(0.8, time);
            (_b = tsample.panner) === null || _b === void 0 ? void 0 : _b.positionX.setValueAtTime(0, time);
            (_c = tsample.panner) === null || _c === void 0 ? void 0 : _c.positionY.setValueAtTime(0, time);
            (_d = tsample.panner) === null || _d === void 0 ? void 0 : _d.positionZ.setValueAtTime(0, time);
            (_e = tsample.panner) === null || _e === void 0 ? void 0 : _e.connect(track);
            tsample.play();
        }
        return () => {
            var _a;
            if (tsample) {
                if (tsample.volume)
                    tsample.volume.gain.exponentialRampToValueAtTime(0.0001, (system.currentTime() + DURATION_CLICK_PAD));
                if (tsample.volume)
                    tsample.volume.gain.value = 0;
                (_a = tsample.panner) === null || _a === void 0 ? void 0 : _a.disconnect(track);
                tsample.destroy();
            }
        };
    };
};
export const loadMod = (url) => {
    return new Promise((resolve, reject) => {
        const headers = new Headers();
        headers.append('User-Agent', 'Andy/1.0');
        headers.append('Accept', 'audio/mod');
        const body = undefined;
        fetch(url, {
            method: 'GET',
            mode: 'cors',
            cache: 'reload',
            headers: headers,
            credentials: 'same-origin',
            redirect: 'follow',
            referrerPolicy: 'no-referrer',
            body: (body) ? JSON.stringify(body) : body,
        })
            .then(response => response.arrayBuffer())
            .then(response => {
            let mod = new MOD(response);
            resolve(mod.song());
        }).catch(e => console.error(e));
    });
};
