import { createPlayHeadClock, toSignature, createTempo } from "../metronome";
import { samplePlayer } from "./audio";
import { NoOp } from "../digital/types";
export function createRun(system, song) {
    if (song.type !== 'M.K.' && song.type !== 'FLT4') {
        throw new Error("Currently only support M.K mod files (4 tracks)");
    }
    const run = {
        song: song,
        timeSignature: toSignature("4/4"),
        currentBpm: 125,
        currentPattern: 0,
        currentRow: 0,
        tracks: [],
        output: system.context().createGain(),
        tickListeners: [],
    };
    for (let i = 0; i < 4; i++) {
        const pan = system.context().createPanner();
        pan.orientationX.setValueAtTime(0, 0);
        pan.orientationY.setValueAtTime(0, 0);
        pan.orientationZ.setValueAtTime(-1, 0);
        pan.positionX.setValueAtTime(0, 0);
        pan.positionY.setValueAtTime(0, 0);
        pan.positionZ.setValueAtTime(0, 0);
        const vol = system.context().createGain();
        pan.connect(vol);
        vol.connect(run.output);
        vol.gain.setValueAtTime(1, 0);
        const t = {
            lastFn: [[NoOp]],
            idx: 0,
            vol,
            pan,
        };
        run.tracks.push(t);
    }
    run.output.connect(system.masterGainNode());
    return run;
}
export function destroyRun(system, run) {
    run.output.disconnect(system.masterGainNode());
}
const flushTracks = (run) => {
    for (let i = 0; i < 4; i++) {
        run.tracks[i].lastFn[0][0]();
        run.tracks[i].lastFn[0][0] = NoOp;
    }
};
const useInstrument = (system, run, which) => {
    return samplePlayer(system, run.song, which);
};
export function insertRun(system, run) {
    let tempo = createTempo(480, 500000);
    run.tickListeners.push((tick, time, bi) => {
        if (bi.beat === 1 && bi.tick === 0) {
        }
        if (bi.tick % 4 === 0) {
            const pos = run.currentRow;
            if (pos < 64) {
                for (let c = 0; c < 4; c++) {
                    const n1 = run.song.patterns[run.currentPattern][run.currentRow][c];
                    if (n1.effect.command === 0xF) {
                        let z = n1.effect.argX * 16 + n1.effect.argY;
                        if (z === 0)
                            z = 1;
                        if (z <= 32) {
                            tempo = createTempo(z * 6, tempo.tempo);
                        }
                        else {
                            console.log("bpm:", z, z + 65);
                        }
                    }
                    if (n1.effect.command === 0xC) {
                        const volume = (n1.effect.argX * 16 + n1.effect.argY) / 64;
                        run.tracks[c].vol.gain.setValueAtTime(volume, time);
                    }
                    if (n1.note !== "") {
                        const instrument = useInstrument(system, run, n1.sample - 1);
                        run.tracks[c].lastFn[0][0]();
                        run.tracks[c].lastFn[0][0] = NoOp;
                        run.tracks[c].lastFn[0][0] = instrument(n1.note, time, run.tracks[c].pan);
                    }
                }
                run.currentRow = pos + 1;
            }
            else if (pos === 64) {
                flushTracks(run);
            }
        }
        return [tempo, run.timeSignature];
    });
    createPlayHeadClock(system, tempo, run.timeSignature, run);
}
