import { BinaryFile } from '../io/file';
import { NoteTable } from './types';
export class MOD {
    constructor(data) {
        this.data = new BinaryFile(data);
    }
    song() {
        const s = {};
        s.title = this.name();
        s.length = this.songLength();
        s.type = this.type();
        s.positions = this.positionTable();
        s.samples = [];
        for (let i = 0; i < 31; i++) {
            s.samples.push(this.sample(i));
        }
        s.sampleBuffers = [];
        const sorted = [...s.positions].sort((a, b) => b - a);
        s.patterns = this.patterns(sorted[0] + 1);
        for (let q = 0; q < s.samples.length; q++) {
            const samp = s.samples[q];
            const audioData = this.data.readSBytes(samp.length);
            s.sampleBuffers.push(audioData);
        }
        return s;
    }
    sample(w = 0) {
        const sample = {};
        const start = 20 + (w * 30);
        this.data.seek(start);
        const name = this.data.readCString();
        sample.name = name;
        this.data.seek(start + 22);
        const len = this.data.readUInt16(false);
        sample.length = len * 2;
        const ft = this.data.readInt8();
        sample.fineTune = (ft & 0x0F) >> 1;
        const vol = this.data.readUInt8();
        sample.volume = vol / 64;
        const rptp = this.data.readUInt16(false);
        sample.repeatPoint = rptp * 2;
        const rptl = this.data.readUInt16(false);
        sample.repeatLength = rptl * 2;
        return sample;
    }
    patterns(num, patternlen = 64, channels = 4) {
        this.data.seek(1084);
        const patterns = new Array(num);
        for (let p = 0; p < num; p++) {
            const pattern = new Array(patternlen);
            for (let rows = 0; rows < patternlen; rows++) {
                const row = new Array(channels);
                for (let channel = 0; channel < channels; channel++) {
                    const uf_np = this.data.readUInt16(false);
                    const lf_ec = this.data.readUInt16(false);
                    const upperSample = (uf_np >> 12) & 0xF;
                    const period = (uf_np & 0x0FFF);
                    const lowerSample = (lf_ec >> 12) & 0xF;
                    const sample = (upperSample << 4) | lowerSample;
                    const effect = (lf_ec & 0x0F00) >> 8;
                    const eargX = (lf_ec & 0x00F0) >> 4;
                    const eargY = (lf_ec & 0x000F);
                    row[channel] = {
                        note: period ? NoteTable.get(period) : "",
                        sample: sample,
                        effect: {
                            command: effect,
                            argX: eargX,
                            argY: eargY,
                        }
                    };
                }
                pattern[rows] = row;
            }
            patterns[p] = pattern;
        }
        return patterns;
    }
    positionTable() {
        const t = [];
        this.data.seek(952);
        for (let i = 0; i < 128; i++) {
            t.push(this.data.readUInt8());
        }
        return t;
    }
    songLength() {
        this.data.seek(950);
        const len = this.data.readUInt8();
        return len;
    }
    type() {
        this.data.seek(1080);
        const tbytes = this.data.readChars(4);
        const str = [];
        for (let z = 0; z < 4; z++) {
            str.push(String.fromCharCode(tbytes[z]));
        }
        return str.join('');
    }
    name() {
        this.data.seek(0);
        const bytes = this.data.readChars(20);
        const str = [];
        for (let z = 0; z < bytes.length; z++) {
            if (bytes[z] === 0x0)
                break;
            str.push(String.fromCharCode(bytes[z]));
        }
        return str.join('');
    }
}
